// EEG RUN ver 1.4
// by Teiji Sawa MD, PhD. 
// Department of Anesthesiology
// Kyoto Prefectural University of Medicine 
// Nov 29, 2020
// Caution: Run this by Processing 2, but not Processing 3!!!
// For Float Class, download FloatTable.pde 
// from Ben Fry's Visualizing Data (https://benfry.com), 
// and put "FloatTable.pde" in the same folder

FloatTable data;
int rowCount;
int columnCount;
int row;
int col;
String field_name[];
int k;
int data_n = 512;

float [] eeg;
String date_now;
int year, month, day, hour, min, sec;
PFont plotFont; 
graphMonitor eegGraph;
PrintWriter output;

void setup() {
    size(600, 300, P2D);
    frameRate(100);
    smooth();
    
    year = year();
    month = month();
    day = day();
    hour = hour();
    min = minute();
    sec = second();
    date_now = year + "_" + nf(month, 2) + "_" + nf(day, 2) + "_" + hour + "-" + nf(min, 2) + "-" + nf(sec, 2);

    row = 0;
    col = 1;
    k = 0;
    
    plotFont = createFont("SansSerif", 20); 
    textFont(plotFont);
    
    eeg = new float [data_n];
     
    data = new FloatTable("user_data/eeg_bis_case1.tsv");
    rowCount = data.getRowCount();
    columnCount = data.getColumnCount();
    field_name = data.getRowNames();
         
    eegGraph = new graphMonitor("EEG", 50, 100, 512, 90);  
    output = createWriter("user_data/" + date_now + "/eeg_run.tsv"); 
}

void draw() {
    background(224);
    fill(0); 
    rectMode(CORNERS); 
    noStroke(); 
    labels();       
    eeg[k] = data.getFloat((row), (col));        
    eegGraph.graphDraw(eeg[k]);     
    output.println(eeg[k]);
    output.flush();
    
    println("Data no.:", (row)*16+(col), "=", eeg[k]);
    println("Row/Total Row:", row, "/", rowCount);  
          
    col  = col + 1;    
    
    if (col == columnCount) {
          row = row + 1;
          col = 1;
    }

    if(k < data_n-1){
          k++;
    }
    else {
          k=0;
    }
          
    if (row == rowCount){ 
           noLoop();
    }          
}

void labels(){
    fill(0);
    textSize(10); 
    textAlign(LEFT, CENTER);
    text("Ver 1.4, Nov 2020 by Teiji Sawa, MD, PhD.", 400, 10);
    textAlign(LEFT, CENTER);
    text("Department of Anesthesiology,", 400, 20);
    textAlign(LEFT, CENTER);
    text("Kyoto Prefectural University of Medicine", 400, 30);

    fill(0);
    textLeading(15);
    textSize(25);
    textAlign(LEFT, CENTER);
    text("EEG RUN", 250, 15);
  
    textSize(10);
    textAlign(LEFT, CENTER);
    text(date_now, 10, 10);
}

void stop() {
  output.close();
  super.stop();
}

class graphMonitor {
    String TITLE;
    int X_POSITION, Y_POSITION;
    int X_LENGTH, Y_LENGTH;
    float [] y1;
    float maxRange;
    graphMonitor(String _TITLE, int _X_POSITION, int _Y_POSITION, int _X_LENGTH, int _Y_LENGTH) {
       TITLE = _TITLE;
       X_POSITION = _X_POSITION;
       Y_POSITION = _Y_POSITION;
       X_LENGTH   = _X_LENGTH;
       Y_LENGTH   = _Y_LENGTH;
       y1 = new float[X_LENGTH];

       for (int i = 0; i < X_LENGTH; i++) {
           y1[i] = 0;
       }
    }

    void graphDraw(float _y1) {
       y1[X_LENGTH - 1] = _y1;
       for (int i = 0; i < X_LENGTH - 1; i++) {
           y1[i] = y1[i + 1];
       }
     
       maxRange = 1;
       for (int i = 0; i < X_LENGTH - 1; i++) {
           maxRange = (abs(y1[i]) > maxRange ? abs(y1[i]) : maxRange);
       }

       pushMatrix();
       translate(X_POSITION, Y_POSITION);
       stroke(130);
       strokeWeight(1);
       rect(0, 0, X_LENGTH, Y_LENGTH);
       line(0, Y_LENGTH / 2, X_LENGTH, Y_LENGTH / 2);
       line(X_LENGTH /4, 0, X_LENGTH /4, Y_LENGTH);
       line(X_LENGTH /2, 0, X_LENGTH/ 2, Y_LENGTH);
       line(X_LENGTH /2  + X_LENGTH /4, 0, X_LENGTH /2  + X_LENGTH /4, Y_LENGTH);
       textSize(12);
       textAlign(RIGHT);
       text(0, -5, Y_LENGTH / 2 + 7);
       text("uV", -7, -5);
       line(0, -5, X_LENGTH, -5);
       
       text("0 sec", 22, -15);
       line(0, -5, 0, -10);
       text("1 sec", X_LENGTH /4 + 22, -15);
       line(X_LENGTH /4, -5, X_LENGTH/4, -10);
       text("2 sec", X_LENGTH /2 + 22, -15);
       line(X_LENGTH /2, -5, X_LENGTH/2, -10);
       text("3 sec", X_LENGTH /2  + X_LENGTH /4 + 22, -15);
       line(X_LENGTH /2  + X_LENGTH /4, -5, X_LENGTH /2  + X_LENGTH /4, -10);
       text("4 sec", X_LENGTH + 22, -15);
       line(X_LENGTH, -5, X_LENGTH, -10);
       
       text(nf(maxRange, 0, 1), -5, 10);
       text(nf(-1 * maxRange, 0, 1), -5, Y_LENGTH);

       translate(X_LENGTH, Y_LENGTH / 2);
       scale(-1, 1);
       strokeWeight(1.5);
       for (int i = 0; i < X_LENGTH - 1; i++) {
           stroke(#00ff00);
           line(i, y1[i] * (Y_LENGTH / 2) / maxRange, i + 1, y1[i + 1] * (Y_LENGTH / 2) / maxRange);
       }
       popMatrix();
   }    
}
