// FFT_run ver 1.3
// by Teiji Sawa MD, PhD. 
// Department of Anesthesiology
// Kyoto Prefectural University of Medicine 
// Nov 29, 2020
// Caution: Run this by Processing 2, but not Processing 3!!!
// For Float Class, download FloatTable.pde 
// from Ben Fry's Visualizing Data (https://benfry.com), 
// and put "FloatTable.pde" in the same folder


import org.apache.commons.math.transform.FastFourierTransformer;
import org.apache.commons.math.complex.Complex;

FloatTable eeg_data;
double x1[];
float x1_Hz[];

Complex y1[];
double y1_abs[];
double y1_dB[];

FastFourierTransformer fft;
int rowCount;

int data_n;

float A_xdataMin; 
float A_xdataMax; 
float A_ydataMax; 
float A_ydataMin;

int A_xvolumeInterval;
int A_yvolumeInterval;
int A_xvolumeIntervalMinor;   
int A_yvolumeIntervalMinor;   

float A_plotX1, A_plotY1; 
float A_plotX2, A_plotY2; 
float A_labelX, A_labelY;

float B_xdataMin; 
float B_xdataMax; 
float B_ydataMax; 
float B_ydataMin;

int B_xvolumeInterval;
int B_yvolumeInterval;
int B_xvolumeIntervalMinor;   
int B_yvolumeIntervalMinor;   

float B_plotX1, B_plotY1; 
float B_plotX2, B_plotY2; 
float B_labelX, B_labelY;

PFont plotFont; 

void setup(){
  size(600, 500, P2D);

  data_n = 1024;

  eeg_data = new FloatTable("data/eeg_bis_10_20_2R.tsv");
  
  x1 = new double [data_n];
  x1_Hz = new float [data_n];
  y1 = new Complex [data_n]; 
  y1_abs = new double [data_n]; 
  y1_dB = new double [data_n];

  fft = new FastFourierTransformer();

  A_xdataMin = 0; 
  A_xdataMax = 47; 
  A_ydataMax = 10; 
  A_ydataMin = 0; 

  A_plotX1 = 100; 
  A_plotX2 = width - A_plotX1+30; 
  A_plotY1 = 30; 
  A_plotY2 = height - A_plotY1-250; 
  A_labelX = 30;
  A_labelY = height - 300;

  A_xvolumeInterval = 10;
  A_yvolumeInterval = 2;
  A_xvolumeIntervalMinor = 5;   
  A_yvolumeIntervalMinor = 1;   

  B_xdataMin = 0; 
  B_xdataMax = 47; 
  B_ydataMax = 80; 
  B_ydataMin = 20; 

  B_plotX1 = 100; 
  B_plotX2 = width - B_plotX1+30; 
  B_plotY1 = 270; 
  B_plotY2 = height - B_plotY1+230; 
  B_labelX = 40;
  B_labelY = height - 100;

  B_xvolumeInterval = 10;
  B_yvolumeInterval = 10;
  B_xvolumeIntervalMinor = 5;   
  B_yvolumeIntervalMinor = 5;   

  plotFont = createFont("SansSerif", 20); 
  textFont(plotFont); 

  frameRate(1);
  smooth(); 
    
}

void draw() {
  background(224);
  fill(0); 
  rectMode(CORNERS); 
  noStroke(); 
  rect(A_plotX1, A_plotY1, A_plotX2, A_plotY2); 
  rect(B_plotX1, B_plotY1, B_plotX2, B_plotY2); 

  A_xdrawVolumeLabels();
  A_ydrawVolumeLabels();
  //A_drawxLabels();
  A_drawyLabels();
  A_drawAxisLabels();

  B_xdrawVolumeLabels();
  B_ydrawVolumeLabels();
  B_drawxLabels();
  B_drawyLabels();
  B_drawAxisLabels();

  get_eegData();
  A_drawPoints();
  B_drawPoints();
}  

void A_xdrawVolumeLabels() {
  fill(0);
  textSize(12);
  textAlign(RIGHT);  
  stroke(128);
  strokeWeight(0.5);
    
  for (float v = A_xdataMin; v <= A_xdataMax; v += A_xvolumeIntervalMinor) {
    if (v % A_xvolumeIntervalMinor == 0) {     // If a tick mark
      float A_x = map(v, A_xdataMax, A_xdataMin, A_plotX2, A_plotX1);  
      float textOffset = textAscent()/2;  // Center vertically
      textOffset = 0;                   // Align by the bottom
      textOffset = textAscent();        // Align by the top
      text(floor(v), A_x + textOffset, A_plotY2 + 20);
      line(A_x, A_plotY2 + 4, A_x, A_plotY2);     // Draw major tick
    }
  }
}

void A_ydrawVolumeLabels() {
  fill(0);
  textSize(14);
  textAlign(RIGHT);  
  stroke(128);
  strokeWeight(0.5);
  for (float v = A_ydataMin; v <= A_ydataMax; v += A_yvolumeIntervalMinor) {
    if (v % A_yvolumeIntervalMinor == 0) {     // If a tick mark
      float A_y = map(v, A_ydataMin, A_ydataMax, A_plotY2, A_plotY1);  
      if (v % A_yvolumeInterval == 0) {        // If a major tick mark
        float textOffset = textAscent()/2;  // Center vertically
        if (v == A_ydataMin) {
          textOffset = 0;                   // Align by the bottom
        } else if (v == A_ydataMax) {
          textOffset = textAscent();        // Align by the top
        }
        text(floor(v), A_plotX1 - 10, A_y + textOffset);
        line(A_plotX1 - 4, A_y, A_plotX1, A_y);     // Draw major tick
      } else {
      line(A_plotX1 - 2, A_y, A_plotX1, A_y);     // Draw minor tick
      }
    }
  }
}

void A_drawxLabels() {
  fill(0);
  stroke(224);
  strokeWeight(0.5);
  
  for (float v = A_xdataMin; v <= A_xdataMax; v += A_xvolumeIntervalMinor) {
    float A_x = map(v, A_xdataMax, A_xdataMin, A_plotX2, A_plotX1);  
    line(A_x, A_plotY1, A_x, A_plotY2);     // Draw major tick
  }
}

void A_drawyLabels() {
  fill(0);
  stroke(224);
  strokeWeight(0.5);
  
  for (float v = A_ydataMin; v <= A_ydataMax; v += A_yvolumeIntervalMinor) {
    if (v % A_yvolumeIntervalMinor == 0) {     // If a tick mark
      float A_y = map(v, A_ydataMin, A_ydataMax, A_plotY2, A_plotY1);  
      if (v % A_yvolumeInterval == 0) {        // If a major tick mark
        line(A_plotX1, A_y, A_plotX2, A_y);     // Draw major tick
      } 
    }
  } 
}

void A_drawAxisLabels() {
  fill(0);
  textSize(14);
  textLeading(15);  
  textAlign(CENTER, CENTER);
  text("uV", A_labelX, (A_plotY1+A_plotY2)/2);
  textAlign(CENTER);
  text("Power Spectrum [Hz]", (A_plotX1+A_plotX2)/2, A_labelY);
}


void B_xdrawVolumeLabels() {
  fill(0);
  textSize(12);
  textAlign(RIGHT);  
  stroke(128);
  strokeWeight(0.5);

  for (float v = B_xdataMin; v <= B_xdataMax; v += B_xvolumeIntervalMinor) {
    if (v % B_xvolumeIntervalMinor == 0) {     // If a tick mark
      float B_x = map(v, B_xdataMax, B_xdataMin, B_plotX2, B_plotX1);  
      float textOffset = textAscent()/2;  // Center vertically
      textOffset = 0;                   // Align by the bottom
      textOffset = textAscent();        // Align by the top
      text(floor(v), B_x + textOffset, B_plotY2 + 20);
      line(B_x, B_plotY2 + 4, B_x, B_plotY2);     // Draw major tick
    }
  }
}

void B_ydrawVolumeLabels() {
  fill(0);
  textSize(14);
  textAlign(RIGHT);  
  stroke(128);
  strokeWeight(0.5);

  for (float v = B_ydataMin; v <= B_ydataMax; v += B_yvolumeIntervalMinor) {
    if (v % B_yvolumeIntervalMinor == 0) {     // If a tick mark
      float B_y = map(v, B_ydataMin, B_ydataMax, B_plotY2, B_plotY1);  
      if (v % B_yvolumeInterval == 0) {        // If a major tick mark
        float textOffset = textAscent()/2;  // Center vertically
        if (v == B_ydataMin) {
          textOffset = 0;                   // Align by the bottom  
        } else if (v == B_ydataMax) {
          textOffset = textAscent();        // Align by the top
        }
        text(floor(v), B_plotX1 - 10, B_y + textOffset);
        line(B_plotX1 - 4, B_y, B_plotX1, B_y);     // Draw major tick
        } else {
          line(B_plotX1 - 2, B_y, B_plotX1, B_y);     // Draw minor tick
        }
    }
  }
}

void B_drawxLabels() {
  fill(0);
  stroke(224);
  strokeWeight(0.5);
  
  for (float v = B_xdataMin; v <= B_xdataMax; v += B_xvolumeIntervalMinor) {
    float B_x = map(v, B_xdataMax, B_xdataMin, B_plotX2, B_plotX1);  
    line(B_x, B_plotY1, B_x, B_plotY2);     // Draw major tick
  }
}

void B_drawyLabels() {
  fill(0);
  stroke(224);
  strokeWeight(0.5);
  
  for (float v = B_ydataMin; v <= B_ydataMax; v += B_yvolumeIntervalMinor) {
    if (v % B_yvolumeIntervalMinor == 0) {     // If a tick mark
      float B_y = map(v, B_ydataMin, B_ydataMax, B_plotY2, B_plotY1);  
      if (v % B_yvolumeInterval == 0) {        // If a major tick mark
        line(B_plotX1, B_y, B_plotX2, B_y);     // Draw major tick
      } 
    }
  } 
}

void B_drawAxisLabels() {
  fill(0);
  textSize(14);
  textLeading(15);  
  textAlign(CENTER, CENTER);
  text("dB", B_labelX, (B_plotY1+B_plotY2)/2);
  textAlign(CENTER);
  text("Power Spectrum [Hz]", (B_plotX1+B_plotX2)/2, B_labelY);
}

void get_eegData(){
  rowCount = eeg_data.getRowCount();
  for (int j = 0; j < 64 ; j++){ 
    for (int i = 0; i < 16 ; i++) {
      x1[j*16+i] = (double) eeg_data.getFloat(rowCount-1-j, i+1); 
      x1_Hz[j*16+i] = 0.125*(j*16+i);
    }
  }

  for (int i=0; i<data_n; i++) {
    println(x1[i]);
  }

  y1 = fft.transform(x1);

  for (int i=0; i<data_n; i++) {
    y1_abs[i] = y1[i].abs(); 
  }

  //for (int i=0; i<data_n; i++) {
  //  y1_abs_Hz[i] = y1_abs[i] ; 
  //}

  for (int i=0; i<data_n; i++) {
    //y1_dB_Hz[i] = 20*log(((float) y1_abs_Hz[i]))/log(10);
    y1_dB[i] = 20*log(((float) y1_abs[i]))/log(10);
    println("y1(dB)=:", y1_dB[i]);
  }
}

void A_drawPoints() { 
  float A_xvalue = 0;
  float A_yvalue = 0;
  float A_x = 0;
  float A_y = 0;
  float A_x_pre = 0;
  float A_y_pre = 0;
   
  for(int i= 0; i<data_n;i++)   {
    A_xvalue = (float) x1_Hz[i];
    //A_yvalue = (float) y1_abs[i] * (float) y1_abs[i] / (data_n/2);
    A_yvalue = (float) y1_abs[i] / (data_n/2);
    
    if (A_xvalue <= A_xdataMax){
      A_x = map(A_xvalue, A_xdataMin, A_xdataMax, A_plotX1, A_plotX2); 
      A_y = map(A_yvalue, A_ydataMin, A_ydataMax, A_plotY2, A_plotY1); 

      strokeWeight(1.5);
      stroke(#00ff00);
      if(i >0){
        line (A_x_pre, A_y_pre, A_x, A_y);
      }
      A_x_pre = A_x;
      A_y_pre = A_y;
    } 
  }
} 

void B_drawPoints() { 
  float B_xvalue = 0;
  float B_yvalue = 0;
  float B_x = 0;
  float B_y = 0;
  float B_x_pre = 0;
  float B_y_pre = 0;
   
  for(int i= 0; i<data_n;i++) { 
    B_xvalue = (float) x1_Hz[i];
    B_yvalue = (float) y1_dB[i];
      
    if (B_xvalue <= B_xdataMax){
      B_x = map(B_xvalue, B_xdataMin, B_xdataMax, B_plotX1, B_plotX2); 
      B_y = map(B_yvalue, B_ydataMin, B_ydataMax, B_plotY2, B_plotY1); 
      strokeWeight(1.5);
      stroke(#00ff00);
      if(i >0){
        line (B_x_pre, B_y_pre, B_x, B_y);
      }
      B_x_pre = B_x;
      B_y_pre = B_y;
    } 
  }
} 
