// SpectrogramMaker ver 1.0
// by Teiji Sawa MD, PhD. 
// Department of Anesthesiology
// Kyoto Prefectural University of Medicine 
// Dev 5, 2020
// For Float Class, download FloatTable.pde 
// from Ben Fry's Visualizing Data (https://benfry.com), 
// and put "FloatTable.pde" in the same folder


String subscript_1 = "Spectrogram Maker, ver 1.0 by T. Sawa, Dec 5, 2020";
String subscript_2 = "Anesthesiology, Kyoro Prefectural University of Medicne";
// ---------------------------------------------

FloatTable data;
PFont plotFont;
int rowCount;
int loop_cnt_for_saveFrame = 0;
int columnCount;
int currentColumn = 0;
int row;
int color_trend_size = 600;
//String freq[];
float Pw[];
float Pw_adj[];
float Pw_adj2[][];
String date_now;
int year, month, day, hour, min, sec;

void setup() {
  size(600, 800, P3D);
  Pw = new float [128];
  Pw_adj = new float [128];
  Pw_adj2 = new float [color_trend_size][128];
   
  row = 1;
  
  array_init_float(Pw);
  array_init_float(Pw_adj);
  array_init_2d_float(Pw_adj2);
  
  data = new FloatTable("data/pw_spectrum_FFT_600.tsv");
  rowCount = data.getRowCount();
  columnCount = data.getColumnCount();
  
  plotFont = createFont("SansSerif", 20);
  textFont(plotFont);

  smooth();

   year = year();
   month = month();
   day = day();
   hour = hour();
   min = minute();
   sec = second();
   
   date_now = year + "_" + nf(month, 2) + "_" + nf(day, 2) + "_" + hour + "-" + nf(min, 2) + "-" + nf(sec, 2);
}

void draw() { 
  colorMode(HSB);
  background(224); 
  graph_labels();

  fill(0); 
  rectMode(CORNERS); 
  noStroke(); 
     
     for (int col = 2; col < columnCount+2; col++) { 
    if (data.isValid(row, col)) { 
      Pw[col-2] = data.getFloat(row, col);      
    }
  }
  
  for (int col = 2; col < columnCount; col++)  {
     Pw_adj[col-2] = Pw[col-2]*12/44-1;
  }
        
    stroke(#0000FF);
    
    power_spectrum();
    array_shift_2d(Pw_adj2, Pw_adj);
        
   loop_cnt_for_saveFrame += 1;
    if (loop_cnt_for_saveFrame == 200) { //200 ループ毎に画面を保存している
         saveFrame("figs/" + date_now + "/frames/########.png"); //8桁でdata フォルダに”日付＋ループ回数.png”で画面がファイル保存される
         loop_cnt_for_saveFrame = 0;
    }
   
   row = row + 1;
   if (row == rowCount+1)
   { 
     noLoop();
   }
 }

void power_spectrum() {
  for (int cnt = 0; cnt < color_trend_size; cnt++) {
    for (int i = 0; i < columnCount-2; i++)  {
    strokeWeight(1);
      noStroke();
      if (Pw_adj2[cnt][i] <= 0) {
         fill(0, 0, 0);
      } else {
        if (floor(255-(int)(Pw_adj2[cnt][i]/12*255))== 255) {
          fill(0, 0, 0);
        } else {
          fill(floor(255-(int)(Pw_adj2[cnt][i]/12*255)), 200, 200);
        }
      }
      rect(80+3.5*i*128/128, 100+cnt, 83.5+3.5*i*128/128, 101+cnt);
    }
  } 
}

void array_shift_2d(float[][] _array1, float[] _array2) {
   for (int j = 1; j<_array1.length; j++) {
     for (int i = 0; i<_array1[j].length; i++) {
       _array1[_array1.length-j][i] = _array1[_array1.length-j-1][i];
       _array1[0][i] = _array2[i];
     }
    }
}

void array_init_float(float[] _array) {
 for (int i=0; i<_array.length; i++) {
    _array[i] = 0;
  }
}

void array_init_2d_float(float[][] _array) {
 for (int j=0; j<_array.length; j++) {
     for (int i=0; i<_array[j].length; i++) {
         _array[j][i] = 0;
    }
   }
 }
 
 void graph_labels() {
   fill(0);
   stroke(40);
   strokeWeight(1);
   line(74, 100, 74, 700);
   line(66, 100, 74, 100);
   line(70, 200, 74, 200);
   line(66, 300, 74, 300);
   line(70, 400, 74, 400);
   line(66, 500, 74, 500);
   line(70, 600, 74, 600);
   line(66, 700, 74, 700);
   textAlign(RIGHT, CENTER);
   textSize(12);
   text("0", 62, 98);
   text("5", 62, 198);
   text("10", 62, 298);
   text("15", 62, 398);
   text("20", 62, 498);
   text("25", 62, 598);
   text("30", 62, 698);
   
   line(80, 94, 524, 94);
   line(80, 86, 81, 94);
   line(151, 86, 152, 94);
   line(224, 86, 225, 94);
   line(296, 86, 297, 94);
   line(369, 86, 370, 94);
   line(441, 86, 442, 94);
   line(513, 86, 514, 94);
   textAlign(CENTER, CENTER);
   textSize(12);
   text("0", 80, 74);
   text("10", 151, 74);
   text("20", 224, 74);
   text("30", 296, 74);
   text("40", 369, 74);
   text("50", 441, 74);
   text("60", 513, 74);
   
   textSize(14);
   textLeading(15);  
   textAlign(LEFT, CENTER);
   text("Hz", 60, 60);
   text("min", 25, 97);
   textSize(18);
   text("Spectrogram", 240, 40);
   
   textAlign(RIGHT, CENTER);
   textSize(12);
   text("Total Data Points:",520, 20);
   text(rowCount, 560, 20);
   text("Analyzing Data:",520, 35);
   text(row, 560, 35);
   text("Time (sec):",520, 50);
   text(row*3, 560, 50);
   
   textSize(11);
   textAlign(LEFT, CENTER);
   text(subscript_1, 280, 775);
   text(subscript_2, 280, 790);
 }
