// SpectrogramMaker_3D ver 1.4
// by Teiji Sawa MD, PhD. 
// Department of Anesthesiology
// Kyoto Prefectural University of Medicine 
// Nov 29, 2020
// For Float Class, download FloatTable.pde 
// from Ben Fry's Visualizing Data (https://benfry.com), 
// and put "FloatTable.pde" in the same folder

import processing.opengl.*;
import org.gwoptics.graphics.*;
import org.gwoptics.graphics.camera.*;
import org.gwoptics.graphics.graph3D.*;
import org.gwoptics.graphics.colourmap.*;
import org.gwoptics.graphics.colourmap.presets.*;

String subscript_1 = "Spectrogram Maker 3D, ver 1.4 by T. Sawa, Nov 29, 2020";
String subscript_2 = "Anesthesiology, Kyoro Prefectural University of Medicne";
// ---------------------------------------------

FloatTable data;
int rowCount;
int loop_cnt_for_saveFrame = 0;
int columnCount;
int currentColumn = 0;
int row;
int color_trend_size = 601;
String freq[];
float Pw[];
float Pw_adj[];
float Pw_adj2[][];
float maxRange40;
String date_now;
int year, month, day, hour, min, sec;
PFont plotFont; 

Camera3D cam;
SurfaceGraph3D g3d;

class standingWave implements IGraph3DCallback{
  public float computePoint(float X, float Y) {
    return (float) surface_plot(X, Y);
  }
}

standingWave gcb = new standingWave();

void setup() {
  size(1000, 800, OPENGL); 

  cam = new Camera3D(this);
  PVector cam_pos = new PVector(300f,300f,-0f);
  cam.setOrthographicView() ;
  cam.getUpVector();
  cam.getRelativePosition();
  cam.setPosition(cam_pos);
  
  plotFont = createFont("font/SansSerif", 20); 
  textFont(plotFont);
  
  Pw = new float [128];
  Pw_adj = new float [128];
  Pw_adj2 = new float [color_trend_size][128];
   
  row = 1;
  array_init_float(Pw);
  array_init_float(Pw_adj);
  array_init_2d_float(Pw_adj2);
  
  data = new FloatTable("data/pw_spectrum_FFT_all_final.tsv");
  rowCount = data.getRowCount();
  columnCount = data.getColumnCount();
  freq = data.getRowNames();
  println(freq[1]);
  fill(0);
  smooth();

   year = year();
   month = month();
   day = day();
   hour = hour();
   min = minute();
   sec = second();
   
   date_now = year + "_" + nf(month, 2) + "_" + nf(day, 2) + "_" + hour + "-" + nf(min, 2) + "-" + nf(sec, 2);

  g3d = new SurfaceGraph3D(this, 400, 600, 250);    
  g3d.setXAxisMin(0);    
  g3d.setXAxisMax(80);
  g3d.setZAxisMin(5);
  //g3d.setZAxisMax(maxRange40);
  g3d.setZAxisMax(12);
  g3d.setYAxisMin(0);    
  g3d.setYAxisMax(600);
  g3d.setAxisColour(63,63,63);
  g3d.setFontColour(63,63,63);  
  g3d.setXAxisMajorTicks(4);
  g3d.setXAxisMinorTicks(4);
  g3d.setYAxisMajorTicks(6);
  g3d.setYAxisMinorTicks(2);
  g3d.setZAxisMajorTicks(3);
  g3d.setXAxisLabelType(ValueType.DECIMAL);
  g3d.setYAxisLabelType(ValueType.DECIMAL);  
  g3d.setZAxisLabelType(ValueType.DECIMAL);  
  // Colourmap presets such as:CoolColourmap, HotColourmap, WarmColourmap or GrayScaleColourmap
 
  g3d.addSurfaceTrace(gcb, 200, 200, new CoolColourmap(true));  
 }

void draw() { 
  colorMode(HSB);
   g3d.plotSurfaceTrace(0);
   background(204);
   noStroke(); 
     
     for (int col = 2; col < columnCount+1; col++) { 
    if (data.isValid(row, col)) { 
      Pw[col-2] = data.getFloat(row, col);      
    }
  }
  maxRange40 = 1;
  for (int col = 2; col < columnCount+1; col++)  {
          maxRange40 = (abs((float)Pw[col-2]) > maxRange40 ? abs((float)Pw[col-2]) : maxRange40);          
  }  
  
  for (int col = 2; col < columnCount+1; col++)  {
     Pw_adj[col-2] = Pw[col-2]*12/maxRange40-1;
     //Pw_adj[col-2] = Pw[col-2]*12/44-1;
  }
            
    array_shift_2d(Pw_adj2, Pw_adj); //array_shift_2d
    
  pushMatrix();
  
  translate(-150,-300,-750);
  
  g3d.draw();
  popMatrix();
    
  loop_cnt_for_saveFrame += 1;
    if (loop_cnt_for_saveFrame == 1) { 
         saveFrame("figs/" + date_now + "/frames/########.png"); 
         loop_cnt_for_saveFrame = 0;
    }
 
   row = row + 1;
   if (row == rowCount+1)
   { 
     noLoop();
   }  
   println("Count/Total Data:", row, "/", rowCount);
 }

float surface_plot(float _x, float _y){      
   int X;
   int Y;
   float Z;

    X = int(_x);
    Y = int(_y);  
    Z = Pw_adj2[Y][X];
    return Z;
}

//void trend_graph() {
//  array_shift_2d(Pw_adj2, Pw_adj);
//}

void array_shift_2d(float[][] _array1, float[] _array2) {
   for (int j = 1; j<_array1.length; j++) {
     for (int i = 0; i<_array1[j].length; i++) {
       _array1[_array1.length-j][i] = _array1[_array1.length-j-1][i];
       _array1[0][i] = _array2[i];
     }
    }
}

void array_init_float(float[] _array) {
 for (int i=0; i<_array.length; i++) {
    _array[i] = 0;
  }
}

void array_init_2d_float(float[][] _array) {
 for (int j=0; j<_array.length; j++) {
     for (int i=0; i<_array[j].length; i++) {
         _array[j][i] = 0;
    }
   }
 }
 
